%%%----------------------------------------------------------------------
%%% File    : foo_sup.erl
%%% Purpose : Top-level supervisor for simple example.
%%% Created : 30 Nov 2002 by Scott Lystig Fritchie <slfritchie@snookles.com>
%%%----------------------------------------------------------------------

-module(foo_sup).
-compile([verbose, report_errors, report_warnings, trace, debug_info]).

-behaviour(supervisor).

%% External exports
-export([start_link/0, start_link/1]).

%% supervisor callbacks
-export([init/1]).

%%%----------------------------------------------------------------------
%%% API
%%%----------------------------------------------------------------------
start_link() ->
    io:format("XXX ~w:start_link()\n", [?MODULE]),
    {error, "Must include StartArgs list"}.

start_link(StartArgs) ->
    io:format("XXX ~w:start_link ArgList = ~w\n", [?MODULE, StartArgs]),
    supervisor:start_link({local, foo_example_sup}, foo_sup, StartArgs).

%%%----------------------------------------------------------------------
%%% Callback functions from supervisor
%%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok,  {SupFlags,  [ChildSpec]}} |
%%          ignore                          |
%%          {error, Reason}   
%%
%% This supervisor will use a one_for_all strategy: if a child dies,
%% all remaining children are killed before restarting the children.
%%----------------------------------------------------------------------

init(ArgList) ->
    io:format("XXX ~w:init ArgList = ~w\n", [?MODULE, ArgList]),
    %% Child_spec = {IdName, {M, F, A},
    %%               Restart, Shutdown_time, Type, Modules_used}
    ArgList1 = ArgList ++ [{register_as, counter1}],
    ArgList2 = ArgList ++ [{register_as, counter2}],
    Counter1 = {counter1, {increment, start_link, [ArgList1]},
		permanent, 2000, worker, [increment]},
    Counter2 = {counter2, {increment, start_link, [ArgList2]},
		permanent, 2000, worker, [increment]},
    %% This will allow up to twenty restarts in a one hour period.
    {ok, {{one_for_all, 20, 3600}, [
				    Counter1,
				    Counter2
				  ]}}.

%%%----------------------------------------------------------------------
%%% Internal functions
%%%----------------------------------------------------------------------

%% None.
